@@ -20,21 +20,17 @@ Provides an actual implementation for the standard commands.
 from time import localtime, strftime
 from datetime import date
 
-import dialogs
-from common import gajim
-from common import helpers
-from common.exceptions import GajimGeneralException
-from common.logger import Constants
+from gajim import dialogs
+from gajim.common import app
+from gajim.common import helpers
+from gajim.common.exceptions import GajimGeneralException
+from gajim.common.logger import KindConstant
 
-from ..errors import CommandError
-from ..framework import CommandContainer, command, doc
-from ..mapping import generate_usage
+from gajim.command_system.errors import CommandError
+from gajim.command_system.framework import CommandContainer, command, doc
+from gajim.command_system.mapping import generate_usage
 
-from hosts import PrivateChatCommands, GroupChatCommands, ChatCommands
-
-# This holds constants fron the logger, which we'll be using in some of our
-# commands.
-lc = Constants()
+from gajim.command_system.implementation.hosts import ChatCommands, PrivateChatCommands, GroupChatCommands
 
 class StandardCommonCommands(CommandContainer):
     """
@@ -45,12 +41,6 @@ class StandardCommonCommands(CommandContainer):
     AUTOMATIC = True
     HOSTS = ChatCommands, PrivateChatCommands, GroupChatCommands
 
-    @command
-    @doc(_("Hide the chat buttons"))
-    def compact(self):
-        new_status = not self.hide_chat_buttons
-        self.chat_buttons_set_visible(new_status)
-
     @command(overlap=True)
     @doc(_("Show help on a given command or a list of available commands if -a is given"))
     def help(self, command=None, all=False):
@@ -91,8 +81,7 @@ class StandardCommonCommands(CommandContainer):
     @command('lastlog', overlap=True)
     @doc(_("Show logged messages which mention given text"))
     def grep(self, text, limit=None):
-        results = gajim.logger.get_search_results_for_query(self.contact.jid,
-                text, self.account)
+        results = app.logger.search_log(self.account, self.contact.jid, text)
 
         if not results:
             raise CommandError(_("%s: Nothing found") % text)
@@ -104,23 +93,22 @@ class StandardCommonCommands(CommandContainer):
                 raise CommandError(_("Limit must be an integer"))
 
         for row in results:
-            contact, time, kind, show, message, subject = row
-
+            contact = row.contact_name
             if not contact:
-                if kind == lc.KIND_CHAT_MSG_SENT:
-                    contact = gajim.nicks[self.account]
+                if row.kind == KindConstant.CHAT_MSG_SENT:
+                    contact = app.nicks[self.account]
                 else:
                     contact = self.contact.name
 
-            time_obj = localtime(time)
-            date_obj = date.fromtimestamp(time)
+            time_obj = localtime(row.time)
+            date_obj = date.fromtimestamp(row.time)
             date_ = strftime('%Y-%m-%d', time_obj)
             time_ = strftime('%H:%M:%S', time_obj)
 
             if date_obj == date.today():
-                formatted = "[%s] %s: %s" % (time_, contact, message)
+                formatted = "[%s] %s: %s" % (time_, contact, row.message)
             else:
-                formatted = "[%s, %s] %s: %s" % (date_, time_, contact, message)
+                formatted = "[%s, %s] %s: %s" % (date_, time_, contact, row.message)
 
             self.echo(formatted)
 
@@ -135,8 +123,8 @@ class StandardCommonCommands(CommandContainer):
     def status(self, status, message):
         if status not in ('online', 'away', 'chat', 'xa', 'dnd'):
             raise CommandError("Invalid status given")
-        for connection in gajim.connections.itervalues():
-            if not gajim.config.get_per('accounts', connection.name,
+        for connection in app.connections.values():
+            if not app.config.get_per('accounts', connection.name,
             'sync_with_global_status'):
                 continue
             if connection.connected < 2:
@@ -149,8 +137,8 @@ class StandardCommonCommands(CommandContainer):
         if not message:
             message = _("Away")
 
-        for connection in gajim.connections.itervalues():
-            if not gajim.config.get_per('accounts', connection.name,
+        for connection in app.connections.values():
+            if not app.config.get_per('accounts', connection.name,
             'sync_with_global_status'):
                 continue
             if connection.connected < 2:
@@ -163,8 +151,8 @@ class StandardCommonCommands(CommandContainer):
         if not message:
             message = _("Available")
 
-        for connection in gajim.connections.itervalues():
-            if not gajim.config.get_per('accounts', connection.name,
+        for connection in app.connections.values():
+            if not app.config.get_per('accounts', connection.name,
             'sync_with_global_status'):
                 continue
             if connection.connected < 2:
@@ -185,17 +173,12 @@ class StandardCommonChatCommands(CommandContainer):
     def clear(self):
         self.conv_textview.clear()
 
-    @command
-    @doc(_("Toggle the OpenPGP encryption"))
-    def gpg(self):
-        self._toggle_gpg()
-
     @command
     @doc(_("Send a ping to the contact"))
     def ping(self):
-        if self.account == gajim.ZEROCONF_ACC_NAME:
+        if self.account == app.ZEROCONF_ACC_NAME:
             raise CommandError(_('Command is not supported for zeroconf accounts'))
-        gajim.connections[self.account].sendPing(self.contact)
+        app.connections[self.account].sendPing(self.contact)
 
     @command
     @doc(_("Send DTMF sequence through an open audio session"))
@@ -282,7 +265,7 @@ class StandardGroupChatCommands(CommandContainer):
     @command('query', raw=True)
     @doc(_("Open a private chat window with a specified occupant"))
     def chat(self, nick):
-        nicks = gajim.contacts.get_nick_list(self.account, self.room_jid)
+        nicks = app.contacts.get_nick_list(self.account, self.room_jid)
         if nick in nicks:
             self.on_send_pm(nick=nick)
         else:
@@ -291,7 +274,7 @@ class StandardGroupChatCommands(CommandContainer):
     @command('msg', raw=True)
     @doc(_("Open a private chat window with a specified occupant and send him a message"))
     def message(self, nick, a_message):
-        nicks = gajim.contacts.get_nick_list(self.account, self.room_jid)
+        nicks = app.contacts.get_nick_list(self.account, self.room_jid)
         if nick in nicks:
             self.on_send_pm(nick=nick, msg=a_message)
         else:
@@ -309,24 +292,16 @@ class StandardGroupChatCommands(CommandContainer):
     @doc(_("Invite a user to a room for a reason"))
     def invite(self, jid, reason):
         self.connection.send_invite(self.room_jid, jid, reason)
-        return _("Invited %s to %s") % (jid, self.room_jid)
+        return _("Invited %(jid)s to %(room_jid)s") % {'jid': jid,
+            'room_jid': self.room_jid}
 
     @command(raw=True, empty=True)
-    @doc(_("Join a group chat given by a jid, optionally using given nickname"))
-    def join(self, jid, nick):
-        if not nick:
-            nick = self.nick
-
+    @doc(_("Join a group chat given by a jid"))
+    def join(self, jid):
         if '@' not in jid:
-            jid = jid + '@' + gajim.get_server_from_jid(self.room_jid)
+            jid = jid + '@' + app.get_server_from_jid(self.room_jid)
 
-        try:
-            gajim.interface.instances[self.account]['join_gc'].window.present()
-        except KeyError:
-            try:
-                dialogs.JoinGroupchatWindow(account=self.account, room_jid=jid, nick=nick)
-            except GajimGeneralException:
-                pass
+        app.interface.join_gc_minimal(self.account, room_jid=jid)
 
     @command('part', 'close', raw=True, empty=True)
     @doc(_("Leave the groupchat, optionally giving a reason, and close tab or window"))
@@ -340,15 +315,15 @@ class StandardGroupChatCommands(CommandContainer):
     If given nickname is not found it will be treated as a jid.
     """))
     def ban(self, who, reason):
-        if who in gajim.contacts.get_nick_list(self.account, self.room_jid):
-            contact = gajim.contacts.get_gc_contact(self.account, self.room_jid, who)
+        if who in app.contacts.get_nick_list(self.account, self.room_jid):
+            contact = app.contacts.get_gc_contact(self.account, self.room_jid, who)
             who = contact.jid
         self.connection.gc_set_affiliation(self.room_jid, who, 'outcast', reason or str())
 
     @command(raw=True, empty=True)
     @doc(_("Kick user by a nick from a groupchat"))
     def kick(self, who, reason):
-        if not who in gajim.contacts.get_nick_list(self.account, self.room_jid):
+        if not who in app.contacts.get_nick_list(self.account, self.room_jid):
             raise CommandError(_("Nickname not found"))
         self.connection.gc_set_role(self.room_jid, who, 'none', reason or str())
 
@@ -360,7 +335,7 @@ class StandardGroupChatCommands(CommandContainer):
     def role(self, who, role):
         if role not in ('moderator', 'participant', 'visitor', 'none'):
             raise CommandError(_("Invalid role given"))
-        if not who in gajim.contacts.get_nick_list(self.account, self.room_jid):
+        if not who in app.contacts.get_nick_list(self.account, self.room_jid):
             raise CommandError(_("Nickname not found"))
         self.connection.gc_set_role(self.room_jid, who, role)
 
@@ -372,17 +347,17 @@ class StandardGroupChatCommands(CommandContainer):
     def affiliate(self, who, affiliation):
         if affiliation not in ('owner', 'admin', 'member', 'outcast', 'none'):
             raise CommandError(_("Invalid affiliation given"))
-        if not who in gajim.contacts.get_nick_list(self.account, self.room_jid):
+        if not who in app.contacts.get_nick_list(self.account, self.room_jid):
             raise CommandError(_("Nickname not found"))
-        contact = gajim.contacts.get_gc_contact(self.account, self.room_jid, who)
+        contact = app.contacts.get_gc_contact(self.account, self.room_jid, who)
         self.connection.gc_set_affiliation(self.room_jid, contact.jid,
             affiliation)
 
     @command
     @doc(_("Display names of all group chat occupants"))
     def names(self, verbose=False):
-        ggc = gajim.contacts.get_gc_contact
-        gnl = gajim.contacts.get_nick_list
+        ggc = app.contacts.get_gc_contact
+        gnl = app.contacts.get_nick_list
 
         get_contact = lambda nick: ggc(self.account, self.room_jid, nick)
         get_role = lambda nick: get_contact(nick).role
@@ -413,8 +388,8 @@ class StandardGroupChatCommands(CommandContainer):
     @command
     @doc(_("Send a ping to the contact"))
     def ping(self, nick):
-        if self.account == gajim.ZEROCONF_ACC_NAME:
+        if self.account == app.ZEROCONF_ACC_NAME:
             raise CommandError(_('Command is not supported for zeroconf accounts'))
-        gc_c = gajim.contacts.get_gc_contact(self.account, self.room_jid, nick)
-        gajim.connections[self.account].sendPing(gc_c, self)
+        gc_c = app.contacts.get_gc_contact(self.account, self.room_jid, nick)
+        app.connections[self.account].sendPing(gc_c, self)
 
