@@ -4,7 +4,7 @@
 ## Copyright (C) 2006 Dimitur Kirov <dkirov AT gmail.com>
 ##                    Travis Shirk <travis AT pobox.com>
 ##                    Nikos Kouremenos <kourem AT gmail.com>
-## Copyright (C) 2006-2017 Yann Leboulanger <asterix AT lagaule.org>
+## Copyright (C) 2006-2014 Yann Leboulanger <asterix AT lagaule.org>
 ##                         Jean-Marie Traissard <jim AT lapin.org>
 ## Copyright (C) 2007 Lukas Petrovicky <lukas AT petrovicky.net>
 ##                    Tomasz Melcer <liori AT exroot.org>
@@ -29,12 +29,12 @@
 ##
 
 try:
-    from common import caps_cache
-    from common.account import Account
-    import common.gajim
-except ImportError, e:
+    from gajim.common import caps_cache
+    from gajim.common.account import Account
+    from gajim import common
+except ImportError as e:
     if __name__ != "__main__":
-        raise ImportError(e)
+        raise ImportError(str(e))
 
 class XMPPEntity(object):
     """
@@ -90,25 +90,27 @@ class Contact(CommonContact):
     """
     Information concerning a contact
     """
-    def __init__(self, jid, account, name='', groups=[], show='', status='',
+    def __init__(self, jid, account, name='', groups=None, show='', status='',
     sub='', ask='', resource='', priority=0, keyID='', client_caps=None,
-    our_chatstate=None, chatstate=None, last_status_time=None, msg_log_id=None,
-    last_activity_time=None):
+    our_chatstate=None, chatstate=None, idle_time=None, avatar_sha=None):
+        if not isinstance(jid, str):
+            print('no str')
+        if groups is None:
+            groups = []
 
         CommonContact.__init__(self, jid, account, resource, show, status, name,
             our_chatstate, chatstate, client_caps=client_caps)
 
         self.contact_name = '' # nick choosen by contact
         self.groups = [i if i else _('General') for i in set(groups)] # filter duplicate values
+        self.avatar_sha = avatar_sha
 
         self.sub = sub
         self.ask = ask
 
         self.priority = priority
         self.keyID = keyID
-        self.msg_log_id = msg_log_id
-        self.last_status_time = last_status_time
-        self.last_activity_time = last_activity_time
+        self.idle_time = idle_time
 
         self.pep = {}
 
@@ -162,8 +164,8 @@ class Contact(CommonContact):
         return is_observer
 
     def is_groupchat(self):
-        for account in common.gajim.gc_connected:
-            if self.jid in common.gajim.gc_connected[account]:
+        for account in common.app.gc_connected:
+            if self.jid in common.app.gc_connected[account]:
                 return True
         return False
 
@@ -179,7 +181,7 @@ class GC_Contact(CommonContact):
 
     def __init__(self, room_jid, account, name='', show='', status='', role='',
     affiliation='', jid='', resource='', our_chatstate=None,
-    chatstate=None):
+    chatstate=None, avatar_sha=None):
 
         CommonContact.__init__(self, jid, account, resource, show, status, name,
             our_chatstate, chatstate)
@@ -187,6 +189,7 @@ class GC_Contact(CommonContact):
         self.room_jid = room_jid
         self.role = role
         self.affiliation = affiliation
+        self.avatar_sha = avatar_sha
 
     def get_full_jid(self):
         return self.room_jid + '/' + self.name
@@ -194,13 +197,16 @@ class GC_Contact(CommonContact):
     def get_shown_name(self):
         return self.name
 
+    def get_avatar(self, *args, **kwargs):
+        return common.app.interface.get_avatar(self.avatar_sha, *args, **kwargs)
+
     def as_contact(self):
         """
         Create a Contact instance from this GC_Contact instance
         """
         return Contact(jid=self.get_full_jid(), account=self.account,
             name=self.name, groups=[], show=self.show, status=self.status,
-            sub='none', client_caps=self.client_caps)
+            sub='none', client_caps=self.client_caps, avatar_sha=self.avatar_sha)
 
 
 class LegacyContactsAPI:
@@ -234,29 +240,30 @@ class LegacyContactsAPI:
         self._metacontact_manager.add_account(account_name)
 
     def get_accounts(self):
-        return self._accounts.keys()
+        return list(self._accounts.keys())
 
     def remove_account(self, account):
         del self._accounts[account]
         self._metacontact_manager.remove_account(account)
 
-    def create_contact(self, jid, account, name='', groups=[], show='',
+    def create_contact(self, jid, account, name='', groups=None, show='',
     status='', sub='', ask='', resource='', priority=0, keyID='',
-    client_caps=None, our_chatstate=None, chatstate=None, last_status_time=None,
-    last_activity_time=None):
+    client_caps=None, our_chatstate=None, chatstate=None, idle_time=None,
+    avatar_sha=None):
+        if groups is None:
+            groups = []
         # Use Account object if available
         account = self._accounts.get(account, account)
         return Contact(jid=jid, account=account, name=name, groups=groups,
             show=show, status=status, sub=sub, ask=ask, resource=resource,
             priority=priority, keyID=keyID, client_caps=client_caps,
             our_chatstate=our_chatstate, chatstate=chatstate,
-            last_status_time=last_status_time,
-            last_activity_time=last_activity_time)
+            idle_time=idle_time, avatar_sha=avatar_sha)
 
     def create_self_contact(self, jid, account, resource, show, status, priority,
     name='', keyID=''):
-        conn = common.gajim.connections[account]
-        nick = name or common.gajim.nicks[account]
+        conn = common.app.connections[account]
+        nick = name or common.app.nicks[account]
         account = self._accounts.get(account, account) # Use Account object if available
         self_contact = self.create_contact(jid=jid, account=account,
                 name=nick, groups=['self_contact'], show=show, status=status,
@@ -280,8 +287,7 @@ class LegacyContactsAPI:
             resource=contact.resource, priority=contact.priority,
             keyID=contact.keyID, client_caps=contact.client_caps,
             our_chatstate=contact.our_chatstate, chatstate=contact.chatstate,
-            last_status_time=contact.last_status_time,
-            last_activity_time=contact.last_activity_time)
+            idle_time=contact.idle_time, avatar_sha=contact.avatar_sha)
 
     def add_contact(self, account, contact):
         if account not in self._accounts:
@@ -304,6 +310,15 @@ class LegacyContactsAPI:
     def get_contact(self, account, jid, resource=None):
         return self._accounts[account].contacts.get_contact(jid, resource=resource)
 
+    def get_avatar(self, account, *args, **kwargs):
+        return self._accounts[account].contacts.get_avatar(*args, **kwargs)
+
+    def get_avatar_sha(self, account, jid):
+        return self._accounts[account].contacts.get_avatar_sha(jid)
+
+    def set_avatar(self, account, jid, sha):
+        self._accounts[account].contacts.set_avatar(jid, sha)
+
     def iter_contacts(self, account):
         for contact in self._accounts[account].contacts.iter_contacts():
             yield contact
@@ -344,20 +359,22 @@ class LegacyContactsAPI:
             return contact
         return self.get_highest_prio_contact_from_contacts(contacts)
 
-    def get_nb_online_total_contacts(self, accounts=[], groups=[]):
+    def get_nb_online_total_contacts(self, accounts=None, groups=None):
         """
         Return the number of online contacts and the total number of contacts
         """
-        if accounts == []:
+        if not accounts:
             accounts = self.get_accounts()
+        if groups is None:
+            groups = []
         nbr_online = 0
         nbr_total = 0
         for account in accounts:
-            our_jid = common.gajim.get_jid_from_account(account)
+            our_jid = common.app.get_jid_from_account(account)
             for jid in self.get_jid_list(account):
                 if jid == our_jid:
                     continue
-                if common.gajim.jid_is_transport(jid) and not \
+                if common.app.jid_is_transport(jid) and not \
                 _('Transports') in groups:
                     # do not count transports
                     continue
@@ -391,10 +408,10 @@ class LegacyContactsAPI:
             raise AttributeError(attr_name)
 
     def create_gc_contact(self, room_jid, account, name='', show='', status='',
-            role='', affiliation='', jid='', resource=''):
+            role='', affiliation='', jid='', resource='', avatar_sha=None):
         account = self._accounts.get(account, account) # Use Account object if available
         return GC_Contact(room_jid, account, name, show, status, role, affiliation, jid,
-                resource)
+                resource, avatar_sha=avatar_sha)
 
     def add_gc_contact(self, account, gc_contact):
         return self._accounts[account].gc_contacts.add_gc_contact(gc_contact)
@@ -420,6 +437,12 @@ class LegacyContactsAPI:
     def get_nb_role_total_gc_contacts(self, account, room_jid, role):
         return self._accounts[account].gc_contacts.get_nb_role_total_gc_contacts(room_jid, role)
 
+    def set_gc_avatar(self, account, room_jid, nick, sha):
+        contact = self.get_gc_contact(account, room_jid, nick)
+        if contact is None:
+            return
+        contact.avatar_sha = sha
+
 
 class Contacts():
     """
@@ -486,24 +509,61 @@ class Contacts():
                     return c
             return self._contacts[jid][0]
 
+    def get_contact_strict(self, jid, resource):
+        """
+        Return the contact instance for the given resource or None
+        """
+        if jid in self._contacts:
+            for c in self._contacts[jid]:
+                if c.resource == resource:
+                    return c
+
+    def get_avatar(self, jid, size=None, scale=None):
+        if jid not in self._contacts:
+            return None
+
+        for resource in self._contacts[jid]:
+            if resource.avatar_sha is None:
+                continue
+            avatar = common.app.interface.get_avatar(
+                resource.avatar_sha, size, scale)
+            if avatar is None:
+                self.set_avatar(jid, None)
+            return avatar
+
+    def get_avatar_sha(self, jid):
+        if jid not in self._contacts:
+            return None
+
+        for resource in self._contacts[jid]:
+            if resource.avatar_sha is not None:
+                return resource.avatar_sha
+        return None
+
+    def set_avatar(self, jid, sha):
+        if jid not in self._contacts:
+            return
+        for resource in self._contacts[jid]:
+            resource.avatar_sha = sha
+
     def iter_contacts(self):
-        for jid in self._contacts.keys():
+        for jid in list(self._contacts.keys()):
             for contact in self._contacts[jid][:]:
                 yield contact
 
     def get_jid_list(self):
-        return self._contacts.keys()
+        return list(self._contacts.keys())
 
     def get_contacts_jid_list(self):
-        return [jid for jid, contact in self._contacts.iteritems() if not
+        return [jid for jid, contact in self._contacts.items() if not
                 contact[0].is_groupchat()]
 
     def get_contact_from_full_jid(self, fjid):
         """
         Get Contact object for specific resource of given jid
         """
-        barejid, resource = common.gajim.get_room_and_nick_from_fjid(fjid)
-        return self.get_contact(barejid, resource)
+        barejid, resource = common.app.get_room_and_nick_from_fjid(fjid)
+        return self.get_contact_strict(barejid, resource)
 
     def get_first_contact_from_jid(self, jid):
         if jid in self._contacts:
@@ -563,7 +623,7 @@ class GC_Contacts():
         gc_list = self.get_gc_list()
         if not room_jid in gc_list:
             return []
-        return self._rooms[room_jid].keys()
+        return list(self._rooms[room_jid].keys())
 
     def get_gc_contact(self, room_jid, nick):
         nick_list = self.get_nick_list(room_jid)
@@ -651,7 +711,7 @@ class MetacontactManager():
             self._metacontacts_tags[brother_account][tag] = [{'jid': brother_jid,
                     'tag': tag}]
             if brother_account != account:
-                common.gajim.connections[brother_account].store_metacontacts(
+                common.app.connections[brother_account].store_metacontacts(
                         self._metacontacts_tags[brother_account])
         # be sure jid has no other tag
         old_tag = self._get_metacontacts_tag(account, jid)
@@ -667,7 +727,7 @@ class MetacontactManager():
             else:
                 self._metacontacts_tags[account][tag].append({'jid': jid,
                         'tag': tag})
-        common.gajim.connections[account].store_metacontacts(
+        common.app.connections[account].store_metacontacts(
                 self._metacontacts_tags[account])
 
     def remove_metacontact(self, account, jid):
@@ -682,7 +742,7 @@ class MetacontactManager():
                     break
             if found:
                 self._metacontacts_tags[account][tag].remove(found)
-                common.gajim.connections[account].store_metacontacts(
+                common.app.connections[account].store_metacontacts(
                         self._metacontacts_tags[account])
                 break
 
@@ -736,88 +796,29 @@ class MetacontactManager():
                     answers.append(data)
         return answers
 
-    def _compare_metacontacts(self, data1, data2):
+    def _metacontact_key(self, data):
         """
-        Compare 2 metacontacts
-
         Data is {'jid': jid, 'account': account, 'order': order} order is
         optional
         """
-        jid1 = data1['jid']
-        jid2 = data2['jid']
-        account1 = data1['account']
-        account2 = data2['account']
-        contact1 = self._contacts.get_contact_with_highest_priority(account1, jid1)
-        contact2 = self._contacts.get_contact_with_highest_priority(account2, jid2)
         show_list = ['not in roster', 'error', 'offline', 'invisible', 'dnd',
-                'xa', 'away', 'chat', 'online', 'requested', 'message']
+                     'xa', 'away', 'chat', 'online', 'requested', 'message']
+
+        jid = data['jid']
+        account = data['account']
         # contact can be null when a jid listed in the metacontact data
         # is not in our roster
-        if not contact1:
-            if contact2:
-                return -1 # prefer the known contact
-            else:
-                show1 = 0
-                priority1 = 0
-        else:
-            show1 = show_list.index(contact1.show)
-            priority1 = contact1.priority
-        if not contact2:
-            if contact1:
-                return 1 # prefer the known contact
-            else:
-                show2 = 0
-                priority2 = 0
-        else:
-            show2 = show_list.index(contact2.show)
-            priority2 = contact2.priority
-        # If only one is offline, it's always second
-        if show1 > 2 and show2 < 3:
-            return 1
-        if show2 > 2 and show1 < 3:
-            return -1
-        if 'order' in data1 and 'order' in data2:
-            if data1['order'] > data2['order']:
-                return 1
-            if data1['order'] < data2['order']:
-                return -1
-        if 'order' in data1:
-            return 1
-        if 'order' in data2:
-            return -1
-        transport1 = common.gajim.get_transport_name_from_jid(jid1)
-        transport2 = common.gajim.get_transport_name_from_jid(jid2)
-        if transport2 and not transport1:
-            return 1
-        if transport1 and not transport2:
-            return -1
-        if show1 > show2:
-            return 1
-        if show2 > show1:
-            return -1
-        if priority1 > priority2:
-            return 1
-        if priority2 > priority1:
-            return -1
-        server1 = common.gajim.get_server_from_jid(jid1)
-        server2 = common.gajim.get_server_from_jid(jid2)
-        myserver1 = common.gajim.config.get_per('accounts', account1, 'hostname')
-        myserver2 = common.gajim.config.get_per('accounts', account2, 'hostname')
-        if server1 == myserver1:
-            if server2 != myserver2:
-                return 1
-        elif server2 == myserver2:
-            return -1
-        if jid1 > jid2:
-            return 1
-        if jid2 > jid1:
-            return -1
-        # If all is the same, compare accounts, they can't be the same
-        if account1 > account2:
-            return 1
-        if account2 > account1:
-            return -1
-        return 0
+        contact = self._contacts.get_contact_with_highest_priority(
+            account, jid)
+        show = show_list.index(contact.show) if contact else 0
+        priority = contact.priority if contact else 0
+        has_order = 'order' in data
+        order = data.get('order', 0)
+        transport = common.app.get_transport_name_from_jid(jid)
+        server = common.app.get_server_from_jid(jid)
+        myserver = common.app.config.get_per('accounts', account, 'hostname')
+        return (bool(contact), show > 2, has_order, order, bool(transport),
+                show, priority, server == myserver, jid, account)
 
     def get_nearby_family_and_big_brother(self, family, account):
         """
@@ -829,7 +830,7 @@ class MetacontactManager():
 
         (nearby_family, big_brother_jid, big_brother_account)
         """
-        if common.gajim.config.get('mergeaccounts'):
+        if common.app.config.get('mergeaccounts'):
             # group all together
             nearby_family = family
         else:
@@ -849,8 +850,7 @@ class MetacontactManager():
         Which of the family will be the big brother under wich all others will be
         ?
         """
-        family.sort(cmp=self._compare_metacontacts)
-        return family[-1]
+        return max(family, key=self._metacontact_key)
 
 
 if __name__ == "__main__":
