@@ -2,7 +2,7 @@
 ## src/common/exceptions.py
 ##
 ## Copyright (C) 2005-2006 Nikos Kouremenos <kourem AT gmail.com>
-## Copyright (C) 2005-2017 Yann Leboulanger <asterix AT lagaule.org>
+## Copyright (C) 2005-2014 Yann Leboulanger <asterix AT lagaule.org>
 ## Copyright (C) 2006 Jean-Marie Traissard <jim AT lapin.org>
 ## Copyright (C) 2007 Brendan Taylor <whateley AT gmail.com>
 ##
@@ -38,11 +38,15 @@ class DatabaseMalformed(Exception):
     The databas can't be read
     """
 
-    def __init__(self):
+    def __init__(self, path=''):
         Exception.__init__(self)
+        self.path = path
 
     def __str__(self):
-        return _('Database cannot be read.')
+        return _('The database file (%s) cannot be read. '
+                 'Try to repair it (see '
+                 'https://dev.gajim.org/gajim/gajim/wikis/help/DatabaseBackup)'
+                 ' or remove it (all history will be lost).') % self.path
 
 class ServiceNotAvailable(Exception):
     """
@@ -76,7 +80,7 @@ class SessionBusNotPresent(Exception):
 
     def __str__(self):
         return _('Session bus is not available.\nTry reading %(url)s') % \
-                {'url': 'http://trac.gajim.org/wiki/GajimDBus'}
+                {'url': 'https://dev.gajim.org/gajim/gajim/wikis/help/GajimDBus'}
 
 class SystemBusNotPresent(Exception):
     """
@@ -88,7 +92,7 @@ class SystemBusNotPresent(Exception):
 
     def __str__(self):
         return _('System bus is not available.\nTry reading %(url)s') % \
-                {'url': 'http://trac.gajim.org/wiki/GajimDBus'}
+                {'url': 'https://dev.gajim.org/gajim/gajim/wikis/help/GajimDBus'}
 
 class NegotiationError(Exception):
     """
@@ -143,3 +147,13 @@ class PluginsystemError(Exception):
 
     def __str__(self):
         return self.text
+
+class StanzaMalformed(Exception):
+    """
+    Malfromed Stanza
+    """
+    def __init__(self, message, stanza=''):
+        Exception.__init__(self, message, stanza)
+        self._msg = '{}\n{}'.format(message, stanza)
+    def __str__(self):
+        return self._msg
