@@ -19,7 +19,6 @@
 ##
 
 import logging
-import i18n
 import os
 import sys
 
@@ -32,7 +31,7 @@ def parseLogLevel(arg):
     elif arg.isupper() and hasattr(logging, arg):
         return getattr(logging, arg)
     else:
-        print _('%s is not a valid loglevel') % repr(arg)
+        print(_('%s is not a valid loglevel') % repr(arg))
         return 0
 
 def parseLogTarget(arg):
@@ -74,7 +73,7 @@ def parseAndSetLogLevels(arg):
             target = parseLogTarget(target.strip())
             if target:
                 logging.getLogger(target).setLevel(level)
-                print "Logger %s level set to %d" % (target, level)
+                print("Logger %s level set to %d" % (target, level))
 
 
 class colors:
@@ -135,7 +134,7 @@ class FancyFormatter(logging.Formatter):
         return logging.Formatter.format(self, record)
 
 
-def init(levels, verbose, quiet):
+def init():
     """
     Iinitialize the logging system
     """
@@ -164,12 +163,8 @@ def init(levels, verbose, quiet):
     root_log.addHandler(consoleloghandler)
     root_log.propagate = False
 
-    if levels:
-        parseAndSetLogLevels(levels)
-    if verbose:
-        set_verbose()
-    if quiet:
-        set_quiet()
+def set_loglevels(loglevels_string):
+    parseAndSetLogLevels(loglevels_string)
 
 def set_verbose():
     parseAndSetLogLevels('gajim=DEBUG')
@@ -182,18 +177,20 @@ def set_quiet():
 
 # tests
 if __name__ == '__main__':
-    init('gajim.c=DEBUG,INFO', None, None)
+    init()
+
+    set_loglevels('gajim.c=DEBUG,INFO')
 
     log = logging.getLogger('gajim')
     log.debug('debug')
     log.info('info')
-    log.warn('warn')
+    log.warning('warn')
     log.error('error')
     log.critical('critical')
 
     log = logging.getLogger('gajim.c.x.dispatcher')
     log.debug('debug')
     log.info('info')
-    log.warn('warn')
+    log.warning('warn')
     log.error('error')
     log.critical('critical')
