@@ -14,9 +14,12 @@
 ## along with Gajim. If not, see <http://www.gnu.org/licenses/>.
 
 import gi
+import logging
 gi.require_version('Gst', '1.0')
 from gi.repository import Gst
 
+log = logging.getLogger('gajim.c.multimedia_helpers')
+
 
 class DeviceManager(object):
     def __init__(self):
@@ -33,12 +36,17 @@ class DeviceManager(object):
     def detect_element(self, name, text, pipe='%s'):
         if Gst.ElementFactory.find(name):
             element = Gst.ElementFactory.make(name, '%spresencetest' % name)
+            if element is None:
+                log.warning('could not create %spresencetest', name)
+                return
             if hasattr(element.props, 'device'):
                 element.set_state(Gst.State.READY)
                 devices = element.get_properties('device')
                 if devices:
                     self.devices[text % _('Default device')] = pipe % name
                     for device in devices:
+                        if device is None:
+                            continue
                         element.set_state(Gst.State.NULL)
                         element.set_property('device', device)
                         element.set_state(Gst.State.READY)
@@ -49,7 +57,7 @@ class DeviceManager(object):
             else:
                 self.devices[text] = pipe % name
         else:
-            print('element \'%s\' not found' % name)
+            log.info('element %s not found', name)
 
 
 class AudioInputManager(DeviceManager):
@@ -108,4 +116,3 @@ class VideoOutputManager(DeviceManager):
         # ximagesink
         self.detect_element('ximagesink', _('X Window System (without Xv)'))
         self.detect_element('autovideosink', _('Autodetect'))
-
