@@ -3,7 +3,7 @@
 ##
 ## Copyright (C) 2006 Santiago Gala
 ##                    Nikos Kouremenos <kourem AT gmail.com>
-## Copyright (C) 2006-2017 Yann Leboulanger <asterix AT lagaule.org>
+## Copyright (C) 2006-2014 Yann Leboulanger <asterix AT lagaule.org>
 ## Copyright (C) 2007 Jean-Marie Traissard <jim AT lapin.org>
 ##
 ## This file is part of Gajim.
@@ -28,7 +28,7 @@ try:
     from docutils import nodes, utils
     from docutils.parsers.rst.roles import set_classes
 except ImportError:
-    print "Requires docutils 0.4 for set_classes to be available"
+    print("Requires docutils 0.4 for set_classes to be available")
     def create_xhtml(text):
         return None
 else:
@@ -62,10 +62,12 @@ else:
                 should return the validated text, or raise ValueError
         """
         def uri_reference_role(role, rawtext, text, lineno, inliner,
-                options={}, content=[]):
+                options=None, content=None):
+            if options is None:
+                options = {}
             try:
                 valid_text = validator(text)
-            except ValueError, e:
+            except ValueError as e:
                 msg = inliner.reporter.error( e.message % dict(text=text), line=lineno)
                 prb = inliner.problematic(rawtext, rawtext, msg)
                 return [prb], [msg]
@@ -92,7 +94,7 @@ else:
                             'XEP #', 'http://www.xmpp.org/extensions/', 'xep-%04d.html',
                             pos_int_validator)
     generate_uri_role('gajim-ticket-reference', ('ticket', 'gtrack'),
-                            'Gajim Ticket #', 'http://trac.gajim.org/ticket/', '%d',
+                            'Gajim Ticket #', 'https://dev.gajim.org/gajim/gajim/issues/', '%d',
                             pos_int_validator)
 
     class HTMLGenerator:
@@ -102,9 +104,10 @@ else:
         It reuses the docutils.core.Publisher class, which means it is *not*
         threadsafe.
         """
-        def __init__(self, settings_spec=None,
-                        settings_overrides=dict(report_level=5, halt_level=5),
-                        config_section='general'):
+        def __init__(self, settings_spec=None, settings_overrides=None,
+                config_section='general'):
+            if settings_overrides is None:
+                settings_overrides = {'report_level': 5, 'halt_level': 5}
             self.pub = Publisher(reader=None, parser=None, writer=None,
                     settings=None,
                     source_class=io.StringInput,
@@ -120,7 +123,7 @@ else:
             # in the JEP
             # &nbsp; ==  u"\u00a0"
             self.pub.writer.translator_class.attribution_formats['dash'] = (
-                    u'\u2014', '')
+                    '\u2014', '')
             self.pub.process_programmatic_settings(settings_spec,
                     settings_overrides,
                     config_section)
@@ -137,7 +140,7 @@ else:
             output = self.pub.publish(enable_exit_status=enable_exit_status)
             # kludge until we can get docutils to stop generating (rare) &nbsp;
             # entities
-            return u'\u00a0'.join(self.pub.writer.parts['fragment'].strip().split(
+            return '\u00a0'.join(self.pub.writer.parts['fragment'].strip().split(
                     '&nbsp;'))
 
     Generator = HTMLGenerator()
@@ -147,7 +150,7 @@ else:
 
 
 if __name__ == '__main__':
-    print "test 1\n", Generator.create_xhtml("""
+    print("test 1\n" + Generator.create_xhtml("""
 test::
 
 >>> print 1
@@ -157,10 +160,10 @@ test::
 
 this `` should    trigger`` should trigger the &nbsp; problem.
 
-""")
-    print "test 2\n", Generator.create_xhtml("""
+"""))
+    print("test 2\n" + Generator.create_xhtml("""
 *test1
 
 test2_
-""")
-    print "test 3\n", Generator.create_xhtml(""":ticket:`316` implements :xep:`71`""")
+"""))
+    print("test 3\n" + Generator.create_xhtml(""":ticket:`316` implements :xep:`71`"""))
