@@ -1,7 +1,7 @@
 # -*- coding:utf-8 -*-
 ## src/common/sleepy.py
 ##
-## Copyright (C) 2003-2017 Yann Leboulanger <asterix AT lagaule.org>
+## Copyright (C) 2003-2014 Yann Leboulanger <asterix AT lagaule.org>
 ## Copyright (C) 2005-2006 Nikos Kouremenos <kourem AT gmail.com>
 ## Copyright (C) 2007 Jean-Marie Traissard <jim AT lapin.org>
 ## Copyright (C) 2008 Mateusz Biliński <mateusz AT bilinski.it>
@@ -21,8 +21,11 @@
 ## along with Gajim. If not, see <http://www.gnu.org/licenses/>.
 ##
 
-from common import gajim
+from gajim.common import app
 import os
+import logging
+
+log = logging.getLogger('gajim.c.sleepy')
 
 
 STATE_UNKNOWN  = 'OS probably not supported'
@@ -49,10 +52,10 @@ try:
         CloseDesktop = ctypes.windll.user32.CloseDesktop
         SystemParametersInfo = ctypes.windll.user32.SystemParametersInfoW
     else: # unix
-        from common import idle
+        from gajim.common import idle
         idle.xss_available
 except Exception:
-    gajim.log.debug('Unable to load idle module')
+    log.debug('Unable to load idle module')
     SUPPORTED = False
 
 class SleepyWindows:
