@@ -3,7 +3,7 @@
 ##
 ## Copyright (C) 2005-2006 Dimitur Kirov <dkirov AT gmail.com>
 ##                         Nikos Kouremenos <kourem AT gmail.com>
-## Copyright (C) 2005-2017 Yann Leboulanger <asterix AT lagaule.org>
+## Copyright (C) 2005-2014 Yann Leboulanger <asterix AT lagaule.org>
 ## Copyright (C) 2006-2008 Jean-Marie Traissard <jim AT lapin.org>
 ## Copyright (C) 2008 Jonathan Schleifer <js-gajim AT webkeks.org>
 ##
@@ -26,6 +26,7 @@ import socket
 import struct
 import hashlib
 import os
+import time
 from errno import EWOULDBLOCK
 from errno import ENOBUFS
 from errno import EINTR
@@ -33,12 +34,11 @@ from errno import EISCONN
 from errno import EINPROGRESS
 from errno import EAFNOSUPPORT
 from nbxmpp.idlequeue import IdleObject
-from file_props import FilesProp
-from common import gajim
-import jingle_xtls
+from gajim.common.file_props import FilesProp
+from gajim.common import app
+from gajim.common import jingle_xtls
 if jingle_xtls.PYOPENSSL_PRESENT:
     import OpenSSL
-    from OpenSSL import SSL
 import logging
 log = logging.getLogger('gajim.c.socks5')
 MAX_BUFF_LEN = 65536
@@ -102,21 +102,21 @@ class SocksQueue:
         if file_props.streamhost_used == True:
             for proxy in file_props.proxyhosts:
                 if proxy['host'] == streamhost['host']:
-                    self.on_success[file_props.sid](proxy)
+                    self.on_success[file_props.transport_sid](proxy)
                     return 1
             return 0
         for host in file_props.streamhosts:
             if streamhost['state'] == 1:
                 return 0
         streamhost['state'] = 1
-        self.on_success[file_props.sid](streamhost)
+        self.on_success[file_props.transport_sid](streamhost)
         return 1
 
-    def connect_to_hosts(self, account, sid, on_success=None, on_failure=None,
-    fingerprint=None, receiving=True):
-        self.on_success[sid] = on_success
-        self.on_failure[sid] = on_failure
-        file_props = FilesProp.getFileProp(account, sid)
+    def connect_to_hosts(self, account, transport_sid, on_success=None,
+    on_failure=None, fingerprint=None, receiving=True):
+        self.on_success[transport_sid] = on_success
+        self.on_failure[transport_sid] = on_failure
+        file_props = FilesProp.getFilePropByTransportSid(account, transport_sid)
         file_props.failure_cb = on_failure
         streamhosts_to_test = []
         # Remove local IPs to not connect to ourself
@@ -125,7 +125,7 @@ class SocksQueue:
                 continue
             streamhosts_to_test.append(streamhost)
         if not streamhosts_to_test:
-            on_failure(file_props.sid)
+            on_failure(file_props.transport_sid)
         # add streamhosts to the queue
         for streamhost in streamhosts_to_test:
             if 'type' in streamhost and streamhost['type'] == 'proxy':
@@ -133,19 +133,23 @@ class SocksQueue:
             else:
                 fp = fingerprint
             if receiving:
-                if 'cid' in streamhost: # jingle
-                    log.debug('Trying to connect as receiver to cid ' + streamhost['cid'])
-                else: # SI
-                    log.debug('Trying to connect as receiver to host ' + streamhost['host'])
+                if 'candidate_id' in streamhost:
+                    log.debug('Trying to connect as receiver to cid ' + \
+                        streamhost['candidate_id'])
+                else:
+                    log.debug('Trying to connect as receiver to jid ' + \
+                        streamhost['jid'])
                 file_props.type_ = 'r'
                 socks5obj = Socks5ReceiverClient(self.idlequeue, streamhost,
-                    sid, file_props, fingerprint=fp)
+                    transport_sid, file_props, fingerprint=fp)
                 self.add_sockobj(account, socks5obj)
             else:
-                if 'cid' in streamhost: #jingle
-                    log.debug('Trying to connect as sender to cid ' + streamhost['cid'])
-                else: # SI
-                    log.debug('Trying to connect as sender to host ' + streamhost['host'])
+                if 'candidate_id' in streamhost:
+                    log.debug('Trying to connect as sender to cid ' + \
+                        streamhost['candidate_id'])
+                else:
+                    log.debug('Trying to connect as sender to jid ' + \
+                        streamhost['jid'])
                 if file_props.sha_str:
                     idx = file_props.sha_str
                 else:
@@ -159,8 +163,8 @@ class SocksQueue:
                 socks5obj = Socks5SenderClient(self.idlequeue, idx,
                     self, _sock=None,host=str(streamhost['host']),
                     port=int(streamhost['port']),fingerprint=fp,
-                    connected=False, file_props=file_props, 
-                    initiator=streamhost['initiator'], 
+                    connected=False, file_props=file_props,
+                    initiator=streamhost['initiator'],
                     target=streamhost['target'])
                 socks5obj.streamhost = streamhost
                 self.add_sockobj(account, socks5obj)
@@ -172,10 +176,10 @@ class SocksQueue:
         Called when there is a host connected to one of the senders's
         streamhosts. Stop other attempts for connections
         """
-        if 'cid' in streamhost: # jingle
-            log.debug('Connected to cid ' + streamhost['cid'])
-        else: # SI
-            log.debug('Connected to host ' + streamhost['host'])
+        if 'candidate_id' in streamhost:
+            log.debug('Connected to cid ' + streamhost['candidate_id'])
+        else:
+            log.debug('Connected to jid ' + streamhost['jid'])
         for host in file_props.streamhosts:
             if host != streamhost and 'idx' in host:
                 if host['state'] == 1:
@@ -219,7 +223,7 @@ class SocksQueue:
                     host['state'] = 0
                     # FIXME: make the sender reconnect also
                     client = Socks5ReceiverClient(self.idlequeue, host,
-                        host['sid'], file_props)
+                        client.sid, file_props)
                     self.add_sockobj(client.account, client)
                     host['idx'] = client.queue_idx
             # we still have chances to connect
@@ -237,10 +241,10 @@ class SocksQueue:
         """
         Called when we loose connection during transfer
         """
-        if 'cid' in streamhost: # jingle
-            log.debug('Connection refused to cid ' + streamhost['cid'])
-        else: # SI
-            log.debug('Connection refused to host ' + streamhost['host'])
+        if 'candidate_id' in streamhost:
+            log.debug('Connection refused to cid ' + streamhost['candidate_id'])
+        else:
+            log.debug('Connection refused to jid ' + streamhost['jid'])
         if file_props is None:
             return
         streamhost['state'] = -1
@@ -252,7 +256,7 @@ class SocksQueue:
         self.readers = {}
         # failure_cb exists - this means that it has never been called
         if file_props.failure_cb:
-            file_props.failure_cb(file_props.sid)
+            file_props.failure_cb(file_props.transport_sid)
             file_props.failure_cb = None
 
     def add_sockobj(self, account, sockobj):
@@ -280,7 +284,7 @@ class SocksQueue:
         '''
         Adds the sockobj to the current list of sockobjects
         '''
-        keys = (file_props.sid, file_props.name, hash_)
+        keys = (file_props.transport_sid, file_props.name, hash_)
         sockobjects[keys] = sockobj
 
     def result_sha(self, sha_str, idx):
@@ -305,7 +309,7 @@ class SocksQueue:
                     sender.file_props.paused = False
                     sender.file_props.stalled = False
                     sender.file_props.elapsed_time = 0
-                    sender.file_props.last_time = self.idlequeue.current_time()
+                    sender.file_props.last_time = time.time()
                     sender.file_props.received_len = 0
                     sender.pauses = 0
                     # start sending file to proxy
@@ -315,14 +319,8 @@ class SocksQueue:
                     self.process_result(result, sender)
 
     def send_file(self, file_props, account, mode):
-        for key in self.senders.keys():
-            if self.senders == {}:
-                # Python acts very weird with this. When there is no keys
-                # in the dictionary It says that it has a key.
-                # Maybe it is my machine. Without this there is a KeyError
-                # traceback.
-                return
-            if file_props.name in key and file_props.sid in key \
+        for key in list(self.senders.keys()):
+            if file_props.name in key and file_props.transport_sid in key \
             and self.senders[key].mode == mode:
                 log.info('socks5: sending file')
                 sender = self.senders[key]
@@ -360,8 +358,8 @@ class SocksQueue:
             sh['initiator'] = None
             sh['target'] = None
             sockobj =  Socks5ReceiverServer(idlequeue=self.idlequeue,
-                streamhost=sh,sid=None, file_props=listener.file_props,
-                fingerprint=None)
+                streamhost=sh,transport_sid=None,
+                file_props=listener.file_props, fingerprint=None)
 
             self._add(sockobj, self.readers, listener.file_props, sock_hash)
             sockobj.set_sock(sock[0])
@@ -400,12 +398,11 @@ class SocksQueue:
         sender = self.senders[key]
         if do_disconnect:
             sender.disconnect()
-        else:
-            self.idlequeue.unplug_idle(sender.fd)
-            self.idlequeue.remove_timeout(sender.fd)
-            del(self.senders[key])
-            if self.connected > 0:
-                self.connected -= 1
+        self.idlequeue.unplug_idle(sender.fd)
+        self.idlequeue.remove_timeout(sender.fd)
+        del(self.senders[key])
+        if self.connected > 0:
+            self.connected -= 1
 
     def remove_receiver(self, idx, do_disconnect=True, remove_all=False):
         """
@@ -413,7 +410,7 @@ class SocksQueue:
         connections with 1
         """
         if idx != -1:
-            for key in self.readers.keys():
+            for key in list(self.readers.keys()):
                 if idx in key:
                     self.remove_receiver_by_key(key,
                         do_disconnect=do_disconnect)
@@ -426,7 +423,7 @@ class SocksQueue:
         connections with 1
         """
         if idx != -1:
-            for key in self.senders.keys():
+            for key in list(self.senders.keys()):
                 if idx in key:
                     self.remove_sender_by_key(key, do_disconnect=do_disconnect)
                     if not remove_all:
@@ -436,19 +433,24 @@ class SocksQueue:
                 self.listener = None
                 self.connected -= 1
 
-    def remove_by_mode(self, sid, mode, do_disconnect=True):
-        for (key, sock) in self.senders.items():
-            if key[0] == sid and sock.mode == mode:
+    def remove_by_mode(self, transport_sid, mode, do_disconnect=True):
+        for (key, sock) in self.senders.copy().items():
+            if key[0] == transport_sid and sock.mode == mode:
                 self.remove_sender_by_key(key)
-        for (key, sock) in self.readers.items():
-            if key[0] == sid and sock.mode == mode:
+        for (key, sock) in self.readers.copy().items():
+            if key[0] == transport_sid and sock.mode == mode:
                 self.remove_receiver_by_key(key)
 
-    def remove_server(self, sid, do_disconnect=True):
-        self.remove_by_mode(sid, 'server')
+    def remove_server(self, transport_sid, do_disconnect=True):
+        self.remove_by_mode(transport_sid, 'server')
 
-    def remove_client(self, sid, do_disconnect=True):
-        self.remove_by_mode(sid, 'client')
+    def remove_client(self, transport_sid, do_disconnect=True):
+        self.remove_by_mode(transport_sid, 'client')
+
+    def remove_other_servers(self, host_to_keep):
+        for (key, sock) in self.senders.copy().items():
+            if sock.host != host_to_keep and sock.mode == 'server':
+                self.remove_sender_by_key(key)
 
 class Socks5(object):
     def __init__(self, idlequeue, host, port, initiator, target, sid):
@@ -470,7 +472,7 @@ class Socks5(object):
         self.state = 0 # not connected
         self.pauses = 0
         self.size = 0
-        self.remaining_buff = ''
+        self.remaining_buff = b''
         self.file = None
         self.connected = False
         self.mode = ''
@@ -497,12 +499,12 @@ class Socks5(object):
         for ai in self.ais:
             try:
                 self._sock = socket.socket(*ai[:3])
-                if not self.fingerprint is None:
+                if self.fingerprint is not None:
                     if self.file_props.type_ == 's':
-                        remote_jid = gajim.get_jid_without_resource(
+                        remote_jid = app.get_jid_without_resource(
                             self.file_props.receiver)
                     else:
-                        remote_jid = gajim.get_jid_without_resource(
+                        remote_jid = app.get_jid_without_resource(
                             self.file_props.sender)
                     self._sock = OpenSSL.SSL.Connection(
                         jingle_xtls.get_context('client',
@@ -512,8 +514,8 @@ class Socks5(object):
                 self._sock.setblocking(False)
                 self._server = ai[4]
                 break
-            except socket.error, e:
-                if not isinstance(e, basestring) and e[0] == EINPROGRESS:
+            except socket.error as e:
+                if e.errno == EINPROGRESS:
                     break
                 # for all other errors, we try other addresses
                 continue
@@ -529,10 +531,10 @@ class Socks5(object):
             self._sock.connect(self._server)
             self._send=self._sock.send
             self._recv=self._sock.recv
-        except (OpenSSL.SSL.WantReadError, OpenSSL.SSL.WantWriteError), e:
+        except (OpenSSL.SSL.WantReadError, OpenSSL.SSL.WantWriteError) as e:
             pass
-        except Exception, ee:
-            errnum = ee[0]
+        except Exception as ee:
+            errnum = ee.errno
             self.connect_timeout += 1
             if errnum == 111 or self.connect_timeout > 1000:
                 self.queue._connection_refused(self.streamhost, self.file_props,
@@ -583,9 +585,9 @@ class Socks5(object):
                     self.size = self.file_props.offset
                     self.file.seek(self.size)
                     self.file_props.received_len = self.size
-            except IOError, e:
+            except IOError as e:
                 self.close_file()
-                raise IOError, e
+                raise IOError(str(e))
 
     def close_file(self):
         # Close file we're sending from
@@ -597,7 +599,7 @@ class Socks5(object):
                     pass
             self.file = None
         # Close file we're receiving into
-        if self.file_props.fd:
+        if self.file_props.fd and self.state >= 7:
             try:
                 self.file_props.fd.close()
             except Exception:
@@ -619,7 +621,7 @@ class Socks5(object):
             fd = open(self.file_props.file_name, opt)
             self.file_props.fd = fd
             self.file_props.elapsed_time = 0
-            self.file_props.last_time = self.idlequeue.current_time()
+            self.file_props.last_time = time.time()
             self.file_props.received_len = offset
         return fd
 
@@ -636,15 +638,15 @@ class Socks5(object):
         Read small chunks of data. Call owner's disconnected() method if
         appropriate
         """
-        received = ''
+        received = b''
         try:
             add = self._recv(64)
         except (OpenSSL.SSL.WantReadError, OpenSSL.SSL.WantWriteError,
-        OpenSSL.SSL.WantX509LookupError), e:
+        OpenSSL.SSL.WantX509LookupError) as e:
             log.info('SSL rehandshake request : ' + repr(e))
             raise e
         except Exception:
-            add = ''
+            add = b''
         received += add
         if len(add) == 0:
             self.disconnect()
@@ -657,20 +659,20 @@ class Socks5(object):
         try:
             self._send(raw_data)
         except (OpenSSL.SSL.WantReadError, OpenSSL.SSL.WantWriteError,
-        OpenSSL.SSL.WantX509LookupError), e:
+        OpenSSL.SSL.WantX509LookupError) as e:
             log.info('SSL rehandshake request :' + repr(e))
             raise e
-        except Exception, e:
+        except Exception:
             self.disconnect()
         return len(raw_data)
 
     def write_next(self):
-        if self.remaining_buff != '':
+        if self.remaining_buff != b'':
             buff = self.remaining_buff
         else:
             try:
                 self.open_file_for_reading()
-            except IOError, e:
+            except IOError:
                 self.state = 8 # end connection
                 self.disconnect()
                 self.file_props.error = -7 # unable to read from file
@@ -681,18 +683,18 @@ class Socks5(object):
             try:
                 lenn = self._send(buff)
             except (OpenSSL.SSL.WantReadError, OpenSSL.SSL.WantWriteError,
-            OpenSSL.SSL.WantX509LookupError), e:
+            OpenSSL.SSL.WantX509LookupError) as e:
                 log.info('SSL rehandshake request :' + repr(e))
                 raise e
-            except Exception, e:
-                if e.args[0] not in (EINTR, ENOBUFS, EWOULDBLOCK):
+            except Exception as e:
+                if e.errno not in (EINTR, ENOBUFS, EWOULDBLOCK):
                     # peer stopped reading
                     self.state = 8 # end connection
                     self.disconnect()
                     self.file_props.error = -1
                     return -1
             self.size += lenn
-            current_time = self.idlequeue.current_time()
+            current_time = time.time()
             self.file_props.elapsed_time += current_time - \
                 self.file_props.last_time
             self.file_props.last_time = current_time
@@ -705,7 +707,7 @@ class Socks5(object):
             if lenn != len(buff):
                 self.remaining_buff = buff[lenn:]
             else:
-                self.remaining_buff = ''
+                self.remaining_buff = b''
             self.state = 7 # continue to write in the socket
             if lenn == 0:
                 return None
@@ -724,21 +726,21 @@ class Socks5(object):
             self.file_props.error = -2
             return None
         fd = None
-        if self.remaining_buff != '':
+        if self.remaining_buff != b'':
             try:
                 fd = self.get_fd()
-            except IOError, e:
-                self.disconnect(False)
+            except IOError:
+                self.disconnect()
                 self.file_props.error = -6 # file system error
                 return 0
             fd.write(self.remaining_buff)
             lenn = len(self.remaining_buff)
-            current_time = self.idlequeue.current_time()
+            current_time = time.time()
             self.file_props.elapsed_time += current_time - \
                 self.file_props.last_time
             self.file_props.last_time = current_time
             self.file_props.received_len += lenn
-            self.remaining_buff = ''
+            self.remaining_buff = b''
             if self.file_props.received_len == self.file_props.size:
                 self.rem_fd(fd)
                 self.disconnect()
@@ -748,19 +750,19 @@ class Socks5(object):
         else:
             try:
                 fd = self.get_fd()
-            except IOError, e:
-                self.disconnect(False)
+            except IOError:
+                self.disconnect()
                 self.file_props.error = -6 # file system error
                 return 0
             try:
                 buff = self._recv(MAX_BUFF_LEN)
             except (OpenSSL.SSL.WantReadError, OpenSSL.SSL.WantWriteError,
-            OpenSSL.SSL.WantX509LookupError), e:
+            OpenSSL.SSL.WantX509LookupError) as e:
                 log.info('SSL rehandshake request :' + repr(e))
                 raise e
             except Exception:
-                buff = ''
-            current_time = self.idlequeue.current_time()
+                buff = b''
+            current_time = time.time()
             self.file_props.elapsed_time += current_time - \
                 self.file_props.last_time
             self.file_props.last_time = current_time
@@ -774,7 +776,7 @@ class Socks5(object):
                 return 0
             try:
                 fd.write(buff)
-            except IOError, e:
+            except IOError:
                 self.rem_fd(fd)
                 self.disconnect()
                 self.file_props.error = -6 # file system error
@@ -834,15 +836,10 @@ class Socks5(object):
         """
         Parse the initial message and create a list of auth mechanisms
         """
-        auth_mechanisms = []
-        try:
-            num_auth = struct.unpack('!xB', buff[:2])[0]
-            for i in xrange(num_auth):
-                mechanism, = struct.unpack('!B', buff[1 + i])
-                auth_mechanisms.append(mechanism)
-        except Exception:
+        if buff[0] != 5:
             return None
-        return auth_mechanisms
+        num_auth = buff[1]
+        return list(buff[2:2+num_auth])
 
     def _get_auth_response(self):
         """
@@ -855,8 +852,8 @@ class Socks5(object):
         Connect request by domain name
         """
         buff = struct.pack('!BBBBB%dsBB' % len(self.host),
-            0x05, 0x01, 0x00, 0x03, len(self.host), self.host, self.port >> 8,
-            self.port & 0xff)
+            0x05, 0x01, 0x00, 0x03, len(self.host), self.host.encode('utf-8'),
+            self.port >> 8, self.port & 0xff)
         return buff
 
     def _get_request_buff(self, msg, command = 0x01):
@@ -864,8 +861,10 @@ class Socks5(object):
         Connect request by domain name, sid sha, instead of domain name (jep
         0096)
         """
-        buff = struct.pack('!BBBBB%dsBB' % len(msg),
-            0x05, command, 0x00, 0x03, len(msg), msg, 0, 0)
+        if isinstance(msg, str):
+            msg = msg.encode('utf-8')
+        buff = struct.pack('!BBBBB%dsBB' % len(msg), 0x05, command, 0x00, 0x03,
+            len(msg), msg, 0, 0)
         return buff
 
     def _parse_request_buff(self, buff):
@@ -876,7 +875,7 @@ class Socks5(object):
                 host, = '.'.join(str(s) for s in host_arr)
                 host_len = len(host)
             elif host_type == 0x03:
-                host_len,  = struct.unpack('!B', buff[4])
+                host_len = buff[4]
                 host, = struct.unpack('!%ds' % host_len, buff[5:5 + host_len])
             portlen = len(buff[host_len + 5:])
             if portlen == 1:
@@ -896,7 +895,7 @@ class Socks5(object):
         Connect response: version, auth method
         """
         try:
-            buff = self._recv()
+            buff = self._recv().decode('utf-8')
         except (OpenSSL.SSL.WantReadError, OpenSSL.SSL.WantWriteError,
         OpenSSL.SSL.WantX509LookupError) as e:
             log.info("SSL rehandshake request : " + repr(e))
@@ -921,11 +920,11 @@ class Socks5(object):
         Get sha of sid + Initiator jid + Target jid
         """
         if self.file_props.is_a_proxy:
-            return hashlib.sha1('%s%s%s' % (self.sid,
-                self.file_props.proxy_sender, self.file_props.proxy_receiver)
-                ).hexdigest()
-        return hashlib.sha1('%s%s%s' % (self.sid, self.initiator,
-            self.target)).hexdigest()
+            return hashlib.sha1(('%s%s%s' % (self.sid,
+                self.file_props.proxy_sender, self.file_props.proxy_receiver)).\
+                encode('utf-8')).hexdigest()
+        return hashlib.sha1(('%s%s%s' % (self.sid, self.initiator,
+            self.target)).encode('utf-8')).hexdigest()
 
 
 class Socks5Sender(IdleObject):
@@ -963,7 +962,7 @@ class Socks5Sender(IdleObject):
         self.file_props.stalled = False
         self.file_props.connected = True
         self.file_props.elapsed_time = 0
-        self.file_props.last_time = self.idlequeue.current_time()
+        self.file_props.last_time = time.time()
         self.file_props.received_len = 0
 
     def start_transfer(self):
@@ -1025,6 +1024,7 @@ class Socks5Receiver(IdleObject):
         self.connect_timeout = 0
         self.connected = False
         self.pauses = 0
+        self.sid = sid
         self.file_props = file_props
         self.file_props.disconnect_cb = self.disconnect
         self.file_props.error = 0
@@ -1050,7 +1050,7 @@ class Socks5Receiver(IdleObject):
         self.file_props.stalled = False
         self.file_props.connected = True
         self.file_props.elapsed_time = 0
-        self.file_props.last_time = self.idlequeue.current_time()
+        self.file_props.last_time = time.time()
         self.file_props.received_len = 0
         self.pauses = 0
         self.state = 7
@@ -1143,7 +1143,7 @@ class Socks5Server(Socks5):
                     result = self.start_transfer() # send
                     self.queue.process_result(result, self)
             except (OpenSSL.SSL.WantReadError, OpenSSL.SSL.WantWriteError,
-            OpenSSL.SSL.WantX509LookupError), e:
+            OpenSSL.SSL.WantX509LookupError) as e:
                 log.info('caught SSL exception, ignored')
         else:
             self.disconnect()
@@ -1181,7 +1181,7 @@ class Socks5Server(Socks5):
             else:
                 self.disconnect()
         except (OpenSSL.SSL.WantReadError, OpenSSL.SSL.WantWriteError,
-        OpenSSL.SSL.WantX509LookupError), e:
+        OpenSSL.SSL.WantX509LookupError) as e:
             log.info('caught SSL exception, ignored')
             return
         if self.state < 5:
@@ -1202,7 +1202,7 @@ class Socks5Client(Socks5):
         """
         result = 1
         buff = self.receive()
-        if buff == '':
+        if buff == b'':
             # end connection
             self.pollend()
             return
@@ -1221,7 +1221,7 @@ class Socks5Client(Socks5):
             version, address_type = struct.unpack('!BxxB', buff[:4])
             addrlen = 0
             if address_type == 0x03:
-                addrlen = ord(buff[4])
+                addrlen = buff[4]
                 address = struct.unpack('!%ds' % addrlen, buff[5:addrlen + 5])
                 portlen = len(buff[addrlen + 5:])
                 if portlen == 1:
@@ -1251,7 +1251,7 @@ class Socks5Client(Socks5):
                 self.file_props.paused = False
                 self.file_props.stalled = False
                 self.file_props.elapsed_time = 0
-                self.file_props.last_time = self.idlequeue.current_time()
+                self.file_props.last_time = time.time()
                 self.file_props.received_len = 0
                 self.pauses = 0
                 # start sending file contents to socket
@@ -1295,7 +1295,7 @@ class Socks5Client(Socks5):
                     result = self.start_transfer() # receive
                     self.queue.process_result(result, self)
             except (OpenSSL.SSL.WantReadError, OpenSSL.SSL.WantWriteError,
-            OpenSSL.SSL.WantX509LookupError), e:
+            OpenSSL.SSL.WantX509LookupError) as e:
                 log.info('caught SSL exception, ignored')
                 return
         else:
@@ -1319,7 +1319,7 @@ class Socks5Client(Socks5):
                 self.queue.process_result(result, self)
                 return
         except (OpenSSL.SSL.WantReadError, OpenSSL.SSL.WantWriteError,
-        OpenSSL.SSL.WantX509LookupError), e:
+        OpenSSL.SSL.WantX509LookupError) as e:
             log.info('caught SSL exception, ignored')
             return
         self.state += 1
@@ -1343,7 +1343,7 @@ class Socks5SenderClient(Socks5Client, Socks5Sender):
     port=None, fingerprint = None, connected=True, file_props=None,
     initiator=None, target=None):
         Socks5Client.__init__(self, idlequeue, host, port, initiator, target,
-            file_props.sid)
+            file_props.transport_sid)
         Socks5Sender.__init__(self,idlequeue, sock_hash, parent,_sock,
             host, port, fingerprint , connected, file_props)
 
@@ -1353,30 +1353,30 @@ class Socks5SenderServer(Socks5Server, Socks5Sender):
     def __init__(self, idlequeue, sock_hash, parent,_sock, host=None,
     port=None, fingerprint = None, connected=True, file_props=None):
         Socks5Server.__init__(self, idlequeue, host, port, None, None,
-            file_props.sid)
+            file_props.transport_sid)
         Socks5Sender.__init__(self,idlequeue, sock_hash, parent, _sock,
             host, port, fingerprint , connected, file_props)
 
 
 class Socks5ReceiverClient(Socks5Client, Socks5Receiver):
-    def __init__(self, idlequeue, streamhost, sid, file_props=None,
+    def __init__(self, idlequeue, streamhost, transport_sid, file_props=None,
     fingerprint=None):
         Socks5Client.__init__(self, idlequeue, streamhost['host'],
             int(streamhost['port']), streamhost['initiator'],
-            streamhost['target'], sid)
-        Socks5Receiver.__init__(self, idlequeue, streamhost, sid, file_props,
-            fingerprint)
+            streamhost['target'], transport_sid)
+        Socks5Receiver.__init__(self, idlequeue, streamhost, transport_sid,
+            file_props, fingerprint)
 
 
 class Socks5ReceiverServer(Socks5Server, Socks5Receiver):
 
-    def __init__(self, idlequeue, streamhost, sid, file_props=None,
+    def __init__(self, idlequeue, streamhost, transport_sid, file_props=None,
     fingerprint=None):
         Socks5Server.__init__(self, idlequeue, streamhost['host'],
             int(streamhost['port']), streamhost['initiator'],
-            streamhost['target'], sid)
-        Socks5Receiver.__init__(self, idlequeue, streamhost, sid, file_props,
-            fingerprint)
+            streamhost['target'], transport_sid)
+        Socks5Receiver.__init__(self, idlequeue, streamhost, transport_sid,
+            file_props, fingerprint)
 
 
 class Socks5Listener(IdleObject):
@@ -1412,8 +1412,8 @@ class Socks5Listener(IdleObject):
                 if self.fingerprint is not None:
                     self._serv = OpenSSL.SSL.Connection(
                         jingle_xtls.get_context('server'), self._serv)
-            except socket.error, e:
-                if e.args[0] == EAFNOSUPPORT:
+            except socket.error as e:
+                if e.errno == EAFNOSUPPORT:
                     self.ai = None
                     continue
                 raise
