@@ -37,6 +37,7 @@ from gi.repository import Pango
 import cairo
 import os
 import sys
+import math
 try:
     from PIL import Image
 except:
@@ -86,9 +87,6 @@ def get_icon_path(icon_name, size=16):
         log.error("Unable to find icon %s: %s" % (icon_name, str(e)))
 
 
-from gajim import dialogs
-
-
 HAS_PYWIN32 = True
 if os.name == 'nt':
     try:
@@ -100,11 +98,10 @@ if os.name == 'nt':
 
 from gajim.common import helpers
 
-def get_display_geometry():
-    display = Gdk.Display.get_default()
-    monitor = display.get_monitor(0)
-    geometry = monitor.get_geometry()
-    return geometry.width, geometry.height
+def get_total_screen_geometry():
+    screen = Gdk.Screen.get_default()
+    window = Gdk.Screen.get_root_window(screen)
+    return window.get_width(), window.get_height()
 
 def add_image_to_button(button, icon_name):
     img = Gtk.Image()
@@ -175,7 +172,7 @@ def move_window(window, x, y):
     """
     Move the window, but also check if out of screen
     """
-    screen_w, screen_h = get_display_geometry()
+    screen_w, screen_h = get_total_screen_geometry()
     if x < 0:
         x = 0
     if y < 0:
@@ -191,7 +188,7 @@ def resize_window(window, w, h):
     """
     Resize window, but also checks if huge window or negative values
     """
-    screen_w, screen_h = get_display_geometry()
+    screen_w, screen_h = get_total_screen_geometry()
     if not w or not h:
         return
     if w > screen_w:
@@ -448,6 +445,7 @@ def scale_with_ratio(size, width, height):
         return size, int(size / ratio)
 
 def on_avatar_save_as_menuitem_activate(widget, avatar, default_name=''):
+    from gajim import dialogs
     def on_continue(response, file_path):
         if response < 0:
             return
@@ -642,10 +640,12 @@ def get_pep_as_pixbuf(pep_class):
     return None
 
 def get_iconset_name_for(name):
+    if name == 'not in roster':
+        name = 'notinroster'
     iconset = app.config.get('iconset')
     if not iconset:
-        return '-'.join([app.config.DEFAULT_ICONSET, name])
-    return '-'.join([iconset, name])
+        iconset = app.config.DEFAULT_ICONSET
+    return '%s-%s' % (iconset, name)
 
 def load_icons_meta():
     """
@@ -879,14 +879,14 @@ def add_css_font():
     unit = "pt" if Gtk.check_version(3, 22, 0) is None else "px"
     css = """
     .font_custom {{
-      font-family: {family};
+      font-family: "{family}";
       font-size: {size}{unit};
       font-weight: {weight};
     }}""".format(
         family=font.get_family(),
         size=int(round(font.get_size() / Pango.SCALE)),
         unit=unit,
-        weight=int(font.get_weight()))
+        weight=pango_to_css_weight(font.get_weight()))
     css = css.replace("font-size: 0{unit};".format(unit=unit), "")
     css = css.replace("font-weight: 0;", "")
     css = "\n".join(filter(lambda x: x.strip(), css.splitlines()))
@@ -911,3 +911,13 @@ def get_image_from_icon_name(icon_name, scale):
     icon = get_iconset_name_for(icon_name)
     surface = gtk_icon_theme.load_surface(icon, 16, scale, None, 0)
     return Gtk.Image.new_from_surface(surface)
+
+def pango_to_css_weight(number):
+    # Pango allows for weight values between 100 and 1000
+    # CSS allows only full hundred numbers like 100, 200 ..
+    number = int(number)
+    if number < 100:
+        return 100
+    if number > 900:
+        return 900
+    return int(math.ceil(number / 100.0)) * 100
