@@ -1,7 +1,7 @@
 # -*- coding:utf-8 -*-
 ## src/session.py
 ##
-## Copyright (C) 2008-2017 Yann Leboulanger <asterix AT lagaule.org>
+## Copyright (C) 2008-2014 Yann Leboulanger <asterix AT lagaule.org>
 ## Copyright (C) 2008 Brendan Taylor <whateley AT gmail.com>
 ##                    Jonathan Schleifer <js-gajim AT webkeks.org>
 ##                    Stephan Erb <steve-e AT h3c.de>
@@ -21,29 +21,32 @@
 ## along with Gajim. If not, see <http://www.gnu.org/licenses/>.
 ##
 
-from common import helpers
+import nbxmpp
 
-from common import events
-from common import exceptions
-from common import gajim
-from common import stanza_session
-from common import contacts
-from common import ged
-from common.connection_handlers_events import ChatstateReceivedEvent, \
+from gajim.common import helpers
+
+from gajim.common import events
+from gajim.common import exceptions
+from gajim.common import app
+from gajim.common import stanza_session
+from gajim.common import contacts
+from gajim.common import ged
+from gajim.common.connection_handlers_events import ChatstateReceivedEvent, \
     InformationEvent
+from gajim.common.logger import KindConstant
 
-import message_control
+from gajim import message_control
 
-import notify
+from gajim import notify
 
-import dialogs
-import negotiation
+from gajim import dialogs
+from gajim import negotiation
 
 class ChatControlSession(stanza_session.EncryptedStanzaSession):
     def __init__(self, conn, jid, thread_id, type_='chat'):
         stanza_session.EncryptedStanzaSession.__init__(self, conn, jid, thread_id,
                 type_='chat')
-        gajim.ged.register_event_handler('decrypted-message-received', ged.GUI1,
+        app.ged.register_event_handler('decrypted-message-received', ged.GUI1,
             self._nec_decrypted_message_received)
 
         self.control = None
@@ -66,10 +69,10 @@ class ChatControlSession(stanza_session.EncryptedStanzaSession):
         """
         if obj.session != self:
             return
-        contact = gajim.contacts.get_contact(self.conn.name, obj.jid,
+        contact = app.contacts.get_contact(self.conn.name, obj.jid,
             obj.resource)
         if not contact:
-            contact = gajim.contacts.get_gc_contact(self.conn.name, obj.jid,
+            contact = app.contacts.get_gc_contact(self.conn.name, obj.jid,
                 obj.resource)
         if self.resource != obj.resource:
             self.resource = obj.resource
@@ -86,45 +89,45 @@ class ChatControlSession(stanza_session.EncryptedStanzaSession):
             if not obj.msgtxt and obj.chatstate is None:
                 return
 
-            log_type = 'chat_msg'
+            log_type = KindConstant.CHAT_MSG_RECV
+            if obj.forwarded and obj.sent:
+                log_type = KindConstant.CHAT_MSG_SENT
         else:
-            log_type = 'single_msg'
-        end = '_recv'
-        if obj.forwarded and obj.sent:
-            end = '_sent'
-        log_type += end
+            log_type = KindConstant.SINGLE_MSG_RECV
+            if obj.forwarded and obj.sent:
+                log_type = KindConstant.SINGLE_MSG_SENT
 
-        if self.is_loggable() and obj.msgtxt:
-            try:
-                if obj.xhtml and gajim.config.get('log_xhtml_messages'):
-                    msg_to_log = obj.xhtml
-                else:
-                    msg_to_log = obj.msgtxt
-                obj.msg_log_id = gajim.logger.write(log_type, obj.fjid,
-                    msg_to_log, tim=obj.timestamp, subject=obj.subject)
-            except exceptions.PysqliteOperationalError, e:
-                gajim.nec.push_incoming_event(InformationEvent(None,
-                    conn=self.conn, level='error', pri_txt=_('Disk Write Error'),
-                    sec_txt=str(e)))
-            except exceptions.DatabaseMalformed:
-                pritext = _('Database Error')
-                sectext = _('The database file (%s) cannot be read. Try to '
-                    'repair it (see http://trac.gajim.org/wiki/DatabaseBackup) '
-                    'or remove it (all history will be lost).') % \
-                    gajim.logger.LOG_DB_PATH
-                gajim.nec.push_incoming_event(InformationEvent(None,
-                    conn=self.conn, level='error', pri_txt=pritext,
-                    sec_txt=sectext))
-
-        treat_as = gajim.config.get('treat_incoming_messages')
+        treat_as = app.config.get('treat_incoming_messages')
         if treat_as:
             obj.mtype = treat_as
         pm = False
-        if obj.gc_control and obj.resource:
+        if obj.muc_pm or (obj.gc_control and obj.resource):
             # It's a Private message
             pm = True
             obj.mtype = 'pm'
 
+        if self.is_loggable() and obj.msgtxt:
+            if obj.xhtml and app.config.get('log_xhtml_messages'):
+                msg_to_log = obj.xhtml
+            else:
+                msg_to_log = obj.msgtxt
+
+            jid = obj.fjid
+            if not pm:
+                jid = obj.jid
+
+            obj.msg_log_id = app.logger.insert_into_logs(
+                self.conn.name, jid, obj.timestamp, log_type,
+                message=msg_to_log,
+                subject=obj.subject,
+                additional_data=obj.additional_data,
+                stanza_id=obj.unique_id)
+
+        if obj.muc_pm and not obj.gc_control:
+            # This is a carbon of a PM from a MUC we are not currently
+            # joined. We log it silently without notification.
+            return True
+
         # Handle chat states
         if contact and (not obj.forwarded or not obj.sent):
             if self.control and self.control.type_id == \
@@ -134,7 +137,7 @@ class ChatControlSession(stanza_session.EncryptedStanzaSession):
                     contact.chatstate = obj.chatstate
                     if contact.our_chatstate == 'ask': # we were jep85 disco?
                         contact.our_chatstate = 'active' # no more
-                    gajim.nec.push_incoming_event(ChatstateReceivedEvent(None,
+                    app.nec.push_incoming_event(ChatstateReceivedEvent(None,
                         conn=obj.conn, msg_obj=obj))
                 elif contact.chatstate != 'active':
                     # got no valid jep85 answer, peer does not support it
@@ -143,20 +146,18 @@ class ChatControlSession(stanza_session.EncryptedStanzaSession):
                 # Brand new message, incoming.
                 contact.our_chatstate = obj.chatstate
                 contact.chatstate = obj.chatstate
-                if obj.msg_log_id: # Do not overwrite an existing msg_log_id with None
-                    contact.msg_log_id = obj.msg_log_id
 
         # THIS MUST BE AFTER chatstates handling
-        # AND BEFORE playsound (else we ear sounding on chatstates!)
+        # AND BEFORE playsound (else we hear sounding on chatstates!)
         if not obj.msgtxt: # empty message text
             return True
 
-        if gajim.config.get_per('accounts', self.conn.name,
-        'ignore_unknown_contacts') and not gajim.contacts.get_contacts(
+        if app.config.get_per('accounts', self.conn.name,
+        'ignore_unknown_contacts') and not app.contacts.get_contacts(
         self.conn.name, obj.jid) and not pm:
             return True
 
-        highest_contact = gajim.contacts.get_contact_with_highest_priority(
+        highest_contact = app.contacts.get_contact_with_highest_priority(
             self.conn.name, obj.jid)
 
         # does this resource have the highest priority of any available?
@@ -165,7 +166,7 @@ class ChatControlSession(stanza_session.EncryptedStanzaSession):
             'offline'
 
         if not self.control:
-            ctrl = gajim.interface.msg_win_mgr.search_control(obj.jid,
+            ctrl = app.interface.msg_win_mgr.search_control(obj.jid,
                 obj.conn.name, obj.resource)
             if ctrl:
                 self.control = ctrl
@@ -179,8 +180,8 @@ class ChatControlSession(stanza_session.EncryptedStanzaSession):
         if not pm:
             self.roster_message2(obj)
 
-        if gajim.interface.remote_ctrl:
-            gajim.interface.remote_ctrl.raise_signal('NewMessage', (
+        if app.interface.remote_ctrl:
+            app.interface.remote_ctrl.raise_signal('NewMessage', (
                 self.conn.name, [obj.fjid, obj.msgtxt, obj.timestamp,
                 obj.encrypted, obj.mtype, obj.subject, obj.chatstate,
                 obj.msg_log_id, obj.user_nick, obj.xhtml, obj.form_node]))
@@ -198,22 +199,22 @@ class ChatControlSession(stanza_session.EncryptedStanzaSession):
         # Try to catch the contact with correct resource
         if resource:
             fjid = jid + '/' + resource
-            contact = gajim.contacts.get_contact(obj.conn.name, jid, resource)
+            contact = app.contacts.get_contact(obj.conn.name, jid, resource)
 
-        highest_contact = gajim.contacts.get_contact_with_highest_priority(
+        highest_contact = app.contacts.get_contact_with_highest_priority(
             obj.conn.name, jid)
         if not contact:
             # If there is another resource, it may be a message from an
             # invisible resource
-            lcontact = gajim.contacts.get_contacts(obj.conn.name, jid)
+            lcontact = app.contacts.get_contacts(obj.conn.name, jid)
             if (len(lcontact) > 1 or (lcontact and lcontact[0].resource and \
             lcontact[0].show != 'offline')) and jid.find('@') > 0:
-                contact = gajim.contacts.copy_contact(highest_contact)
+                contact = app.contacts.copy_contact(highest_contact)
                 contact.resource = resource
                 contact.priority = 0
                 contact.show = 'offline'
                 contact.status = ''
-                gajim.contacts.add_contact(obj.conn.name, contact)
+                app.contacts.add_contact(obj.conn.name, contact)
 
             else:
                 # Default to highest prio
@@ -222,11 +223,11 @@ class ChatControlSession(stanza_session.EncryptedStanzaSession):
 
         if not contact:
             # contact is not in roster
-            contact = gajim.interface.roster.add_to_not_in_the_roster(
+            contact = app.interface.roster.add_to_not_in_the_roster(
                 obj.conn.name, jid, obj.user_nick)
 
         if not self.control:
-            ctrl = gajim.interface.msg_win_mgr.search_control(obj.jid,
+            ctrl = app.interface.msg_win_mgr.search_control(obj.jid,
                 obj.conn.name, obj.resource)
             if ctrl:
                 self.control = ctrl
@@ -247,18 +248,30 @@ class ChatControlSession(stanza_session.EncryptedStanzaSession):
             # We have a ChatControl open
             obj.show_in_roster = False
             obj.show_in_systray = False
+            do_event = False
         elif obj.forwarded and obj.sent:
             # Its a Carbon Copied Message we sent
             obj.show_in_roster = False
             obj.show_in_systray = False
-            gajim.events.remove_events(self.conn.name, fjid, types=['chat'])
+            unread_events = app.events.get_events(
+                self.conn.name, fjid, types=['chat'])
+            read_ids = []
+            for msg in unread_events:
+                read_ids.append(msg.msg_log_id)
+            app.logger.set_read_messages(read_ids)
+            app.events.remove_events(self.conn.name, fjid, types=['chat'])
+            do_event = False
         else:
             # Everything else
             obj.show_in_roster = notify.get_show_in_roster(event_type,
                 self.conn.name, contact, self)
             obj.show_in_systray = notify.get_show_in_systray(event_type,
                 self.conn.name, contact)
-
+            if obj.mtype == 'normal' and obj.popup:
+                do_event = False
+            else:
+                do_event = True
+        if do_event:
             event = event_t(obj.msgtxt, obj.subject, obj.mtype, obj.timestamp,
                 obj.encrypted, obj.resource, obj.msg_log_id,
                 correct_id=(obj.id_, obj.correct_id), xhtml=obj.xhtml,
@@ -266,40 +279,44 @@ class ChatControlSession(stanza_session.EncryptedStanzaSession):
                 displaymarking=obj.displaymarking,
                 sent_forwarded=obj.forwarded and obj.sent,
                 show_in_roster=obj.show_in_roster,
-                show_in_systray=obj.show_in_systray)
+                show_in_systray=obj.show_in_systray,
+                additional_data=obj.additional_data)
 
-            gajim.events.add_event(self.conn.name, fjid, event)
+            app.events.add_event(self.conn.name, fjid, event)
 
     def roster_message(self, jid, msg, tim, encrypted=False, msg_type='',
     subject=None, resource='', msg_log_id=None, user_nick='', xhtml=None,
-    form_node=None, displaymarking=None):
+    form_node=None, displaymarking=None, additional_data=None):
         """
         Display the message or show notification in the roster
         """
         contact = None
         fjid = jid
 
+        if additional_data is None:
+            additional_data = {}
+
         # Try to catch the contact with correct resource
         if resource:
             fjid = jid + '/' + resource
-            contact = gajim.contacts.get_contact(self.conn.name, jid, resource)
+            contact = app.contacts.get_contact(self.conn.name, jid, resource)
 
-        highest_contact = gajim.contacts.get_contact_with_highest_priority(
+        highest_contact = app.contacts.get_contact_with_highest_priority(
                 self.conn.name, jid)
         if not contact:
             # If there is another resource, it may be a message from an invisible
             # resource
-            lcontact = gajim.contacts.get_contacts(self.conn.name, jid)
+            lcontact = app.contacts.get_contacts(self.conn.name, jid)
             if (len(lcontact) > 1 or (lcontact and lcontact[0].resource and \
             lcontact[0].show != 'offline')) and jid.find('@') > 0:
-                contact = gajim.contacts.copy_contact(highest_contact)
+                contact = app.contacts.copy_contact(highest_contact)
                 contact.resource = resource
                 if resource:
                     fjid = jid + '/' + resource
                 contact.priority = 0
                 contact.show = 'offline'
                 contact.status = ''
-                gajim.contacts.add_contact(self.conn.name, contact)
+                app.contacts.add_contact(self.conn.name, contact)
 
             else:
                 # Default to highest prio
@@ -308,11 +325,11 @@ class ChatControlSession(stanza_session.EncryptedStanzaSession):
 
         if not contact:
             # contact is not in roster
-            contact = gajim.interface.roster.add_to_not_in_the_roster(
+            contact = app.interface.roster.add_to_not_in_the_roster(
                     self.conn.name, jid, user_nick)
 
         if not self.control:
-            ctrl = gajim.interface.msg_win_mgr.get_control(fjid, self.conn.name)
+            ctrl = app.interface.msg_win_mgr.get_control(fjid, self.conn.name)
             if ctrl:
                 self.control = ctrl
                 self.control.set_session(self)
@@ -320,7 +337,7 @@ class ChatControlSession(stanza_session.EncryptedStanzaSession):
                 fjid = jid
 
         # Do we have a queue?
-        no_queue = len(gajim.events.get_events(self.conn.name, fjid)) == 0
+        no_queue = len(app.events.get_events(self.conn.name, fjid)) == 0
 
         popup = helpers.allow_popup_window(self.conn.name)
 
@@ -338,10 +355,11 @@ class ChatControlSession(stanza_session.EncryptedStanzaSession):
                 typ = 'error'
 
             self.control.print_conversation(msg, typ, tim=tim, encrypted=encrypted,
-                    subject=subject, xhtml=xhtml, displaymarking=displaymarking)
+                    subject=subject, xhtml=xhtml, displaymarking=displaymarking,
+                    additional_data=additional_data)
 
             if msg_log_id:
-                gajim.logger.set_read_messages([msg_log_id])
+                app.logger.set_read_messages([msg_log_id])
 
             return
 
@@ -361,33 +379,33 @@ class ChatControlSession(stanza_session.EncryptedStanzaSession):
         event = event_t(msg, subject, msg_type, tim, encrypted, resource,
             msg_log_id, xhtml=xhtml, session=self, form_node=form_node,
             displaymarking=displaymarking, sent_forwarded=False,
-            show_in_roster=show_in_roster,
-            show_in_systray=show_in_systray)
+            show_in_roster=show_in_roster, show_in_systray=show_in_systray,
+            additional_data=additional_data)
 
-        gajim.events.add_event(self.conn.name, fjid, event)
+        app.events.add_event(self.conn.name, fjid, event)
 
         if popup:
             if not self.control:
-                self.control = gajim.interface.new_chat(contact,
+                self.control = app.interface.new_chat(contact,
                     self.conn.name, session=self)
 
-                if len(gajim.events.get_events(self.conn.name, fjid)):
+                if len(app.events.get_events(self.conn.name, fjid)):
                     self.control.read_queue()
         else:
             if no_queue: # We didn't have a queue: we change icons
-                gajim.interface.roster.draw_contact(jid, self.conn.name)
+                app.interface.roster.draw_contact(jid, self.conn.name)
 
-            gajim.interface.roster.show_title() # we show the * or [n]
+            app.interface.roster.show_title() # we show the * or [n]
         # Select the big brother contact in roster, it's visible because it has
         # events.
-        family = gajim.contacts.get_metacontacts_family(self.conn.name, jid)
+        family = app.contacts.get_metacontacts_family(self.conn.name, jid)
         if family:
             nearby_family, bb_jid, bb_account = \
-                    gajim.contacts.get_nearby_family_and_big_brother(family,
+                    app.contacts.get_nearby_family_and_big_brother(family,
                     self.conn.name)
         else:
             bb_jid, bb_account = jid, self.conn.name
-        gajim.interface.roster.select_contact(bb_jid, bb_account)
+        app.interface.roster.select_contact(bb_jid, bb_account)
 
     # ---- ESessions stuff ---
 
@@ -448,7 +466,7 @@ class ChatControlSession(stanza_session.EncryptedStanzaSession):
             'submit':
                 try:
                     self.archiving_accepted(form)
-                except exceptions.NegotiationError, details:
+                except exceptions.NegotiationError as details:
                     self.fail_bad_negotiation(details)
 
                 return
@@ -464,27 +482,27 @@ class ChatControlSession(stanza_session.EncryptedStanzaSession):
                             dialog.destroy()
 
                         if is_checked:
-                            allow_no_log_for = gajim.config.get_per(
+                            allow_no_log_for = app.config.get_per(
                                 'accounts', self.conn.name,
                                 'allow_no_log_for').split()
                             jid = str(self.jid)
                             if jid not in allow_no_log_for:
                                 allow_no_log_for.append(jid)
-                                gajim.config.set_per('accounts', self.conn.name,
+                                app.config.set_per('accounts', self.conn.name,
                                 'allow_no_log_for', ' '.join(allow_no_log_for))
 
                         negotiated.update(ask_user)
 
                         try:
                             self.accept_e2e_alice(form, negotiated)
-                        except exceptions.NegotiationError, details:
+                        except exceptions.NegotiationError as details:
                             self.fail_bad_negotiation(details)
 
                     def reject_nondefault_options():
                         self.reject_negotiation()
                         dialog.destroy()
 
-                    allow_no_log_for = gajim.config.get_per('accounts',
+                    allow_no_log_for = app.config.get_per('accounts',
                         self.conn.name, 'allow_no_log_for').split()
                     if str(self.jid) in allow_no_log_for:
                         dialog = None
@@ -502,7 +520,7 @@ class ChatControlSession(stanza_session.EncryptedStanzaSession):
                 else:
                     try:
                         self.accept_e2e_alice(form, negotiated)
-                    except exceptions.NegotiationError, details:
+                    except exceptions.NegotiationError as details:
                         self.fail_bad_negotiation(details)
 
                 return
@@ -510,21 +528,21 @@ class ChatControlSession(stanza_session.EncryptedStanzaSession):
             'result':
                 try:
                     self.we_accept_archiving(form)
-                except exceptions.NegotiationError, details:
+                except exceptions.NegotiationError as details:
                     self.fail_bad_negotiation(details)
 
                 return
             elif self.status == 'responded-e2e' and form.getType() == 'result':
                 try:
                     self.accept_e2e_bob(form)
-                except exceptions.NegotiationError, details:
+                except exceptions.NegotiationError as details:
                     self.fail_bad_negotiation(details)
 
                 return
             elif self.status == 'identified-alice' and form.getType() == 'result':
                 try:
                     self.final_steps_alice(form)
-                except exceptions.NegotiationError, details:
+                except exceptions.NegotiationError as details:
                     self.fail_bad_negotiation(details)
 
                 return
@@ -547,17 +565,17 @@ class ChatControlSession(stanza_session.EncryptedStanzaSession):
         # around to test my test suite.
         if form.getType() == 'form':
             if not self.control:
-                jid, resource = gajim.get_room_and_nick_from_fjid(str(self.jid))
+                jid, resource = app.get_room_and_nick_from_fjid(str(self.jid))
 
                 account = self.conn.name
-                contact = gajim.contacts.get_contact(account, str(self.jid),
+                contact = app.contacts.get_contact(account, str(self.jid),
                     resource)
 
                 if not contact:
-                    contact = gajim.contacts.create_contact(jid=jid, account=account,
+                    contact = app.contacts.create_contact(jid=jid, account=account,
                             resource=resource, show=self.conn.get_status())
 
-                gajim.interface.new_chat(contact, account, resource=resource,
+                app.interface.new_chat(contact, account, resource=resource,
                         session=self)
 
             negotiation.FeatureNegotiationWindow(account, str(self.jid), self,
